@@ -59,7 +59,6 @@
 #define KEY_CUSTOM_COMMAND "custom_command"
 #define KEY_ICON "icon"
 #define KEY_PALETTE "palette"
-#define KEY_X_FONT "x_font"
 #define KEY_BACKGROUND_TYPE "background_type"
 #define KEY_BACKGROUND_IMAGE "background_image"
 #define KEY_SCROLL_BACKGROUND "scroll_background"
@@ -103,8 +102,6 @@ struct _TerminalProfilePrivate
 
   GdkColor palette[TERMINAL_PALETTE_SIZE];
 
-  char *x_font;
-
   TerminalBackgroundType background_type;
   char *background_image_file;
   GdkPixbuf *background_image;
@@ -285,7 +282,6 @@ terminal_profile_init (TerminalProfile *profile)
   memcpy (profile->priv->palette,
           terminal_palette_linux,
           TERMINAL_PALETTE_SIZE * sizeof (GdkColor));
-  profile->priv->x_font = g_strdup ("fixed");
   profile->priv->background_type = TERMINAL_BACKGROUND_SOLID;
   profile->priv->background_image_file = g_strdup ("");
   profile->priv->background_darkness = 0.0;
@@ -352,7 +348,6 @@ terminal_profile_finalize (GObject *object)
   g_free (profile->priv->icon_file);
   if (profile->priv->icon)
     g_object_unref (G_OBJECT (profile->priv->icon));
-  g_free (profile->priv->x_font);
 
   g_free (profile->priv->background_image_file);
   if (profile->priv->background_image)
@@ -1125,33 +1120,6 @@ terminal_profile_set_palette (TerminalProfile *profile,
   g_free (str);
 }
 
-const char*
-terminal_profile_get_x_font (TerminalProfile *profile)
-{
-  g_return_val_if_fail (TERMINAL_IS_PROFILE (profile), NULL);
-
-  return profile->priv->x_font;
-}
-
-void
-terminal_profile_set_x_font (TerminalProfile *profile,
-                             const char      *name)
-{
-  char *key;
-
-  RETURN_IF_NOTIFYING (profile);
-  
-  key = gconf_concat_dir_and_key (profile->priv->profile_dir,
-                                  KEY_X_FONT);
-  
-  gconf_client_set_string (profile->priv->conf,
-                           key,
-                           name,
-                           NULL);
-
-  g_free (key);
-}
-
 TerminalBackgroundType
 terminal_profile_get_background_type (TerminalProfile *profile)
 {
@@ -1774,25 +1742,6 @@ set_palette (TerminalProfile *profile,
   return FALSE;
 }
 
-static gboolean
-set_x_font (TerminalProfile *profile,
-            const char      *candidate_font)
-{
-  if (candidate_font &&
-      strcmp (profile->priv->x_font, candidate_font) == 0)
-    return FALSE;
-  
-  if (candidate_font != NULL)
-    {
-      g_free (profile->priv->x_font);
-      profile->priv->x_font = g_strdup (candidate_font);
-      return TRUE;
-    }
-  /* otherwise just leave the old font */
-  
-  return FALSE;
-}
-
 static gboolean
 set_background_type (TerminalProfile *profile,
                      const char      *str_val)
@@ -2008,7 +1957,6 @@ terminal_profile_update (TerminalProfile *profile)
   UPDATE_STRING  (KEY_CUSTOM_COMMAND,       custom_command);
   UPDATE_STRING  (KEY_ICON,                 icon_file);
   UPDATE_STRING  (KEY_PALETTE,              palette);
-  UPDATE_STRING  (KEY_X_FONT,               x_font);
   UPDATE_STRING  (KEY_BACKGROUND_TYPE,      background_type);
   UPDATE_STRING  (KEY_BACKGROUND_IMAGE,     background_image_file);
   UPDATE_BOOLEAN (KEY_SCROLL_BACKGROUND,    scroll_background);
@@ -2155,7 +2103,6 @@ else if (strcmp (key, KName) == 0)                                      \
      UPDATE_STRING  (KEY_CUSTOM_COMMAND,         custom_command,         NULL);
      UPDATE_STRING  (KEY_ICON,                   icon_file,              NULL);
      UPDATE_STRING  (KEY_PALETTE,                palette,                NULL);
-     UPDATE_STRING  (KEY_X_FONT,                 x_font,                 NULL);
      UPDATE_STRING  (KEY_BACKGROUND_TYPE,        background_type,        NULL);
      UPDATE_STRING  (KEY_BACKGROUND_IMAGE,       background_image_file,  NULL);
      UPDATE_BOOLEAN (KEY_SCROLL_BACKGROUND,      scroll_background,      FALSE);
@@ -2512,7 +2459,7 @@ emit_changed (TerminalProfile           *profile,
 }
 
 /* Function I'm cut-and-pasting everywhere, this is from msm */
-void
+static void
 dialog_add_details (GtkDialog  *dialog,
                     const char *details)
 {
@@ -2831,14 +2778,6 @@ terminal_profile_create (TerminalProfile *base_profile,
   g_free (s);
   BAIL_OUT_CHECK ();
 
-  g_free (key);
-  key = gconf_concat_dir_and_key (profile_dir,
-                                  KEY_X_FONT);
-  gconf_client_set_string (base_profile->priv->conf,
-                           key, base_profile->priv->x_font,
-                           &err);
-  BAIL_OUT_CHECK ();
-
   g_free (key);
   key = gconf_concat_dir_and_key (profile_dir,
                                   KEY_BACKGROUND_TYPE);
@@ -3120,7 +3059,7 @@ terminal_profile_get_for_new_term (TerminalProfile *current)
  * does not carry all the bytes, and xterm's palette is messed up...
  */
 
-gchar*
+static gchar*
 color_selection_palette_to_string (const GdkColor *colors, gint n_colors)
 {
   gint i;
