@@ -24,7 +24,6 @@
 #include "terminal-accels.h"
 #include "terminal-widget.h"
 #include "terminal-window.h"
-#include "terminal-notebook.h"
 #include "terminal.h"
 #include "encoding.h"
 #include <string.h>
@@ -69,7 +68,7 @@ struct _TerminalWindowPrivate
   GtkWidget *go_menu;
   GtkWidget *encoding_menuitem;
   GList *tab_menuitems;
-  GList *terms;
+  guint terms;
   TerminalScreen *active_term;
   GdkPixbuf *icon;
   GtkClipboard *clipboard;
@@ -92,6 +91,9 @@ struct _TerminalWindowPrivate
 
   /* Compositing manager integration */
   guint have_argb_visual : 1;
+
+  guint disposed : 1;
+  guint present_on_insert : 1;
 };
 
 enum {
@@ -99,13 +101,10 @@ enum {
   LAST_SIGNAL
 };
 
-static GConfClient *client = NULL;
-static guint notify;
-
 static void terminal_window_init        (TerminalWindow      *window);
 static void terminal_window_class_init  (TerminalWindowClass *klass);
+static void terminal_window_dispose     (GObject             *object);
 static void terminal_window_finalize    (GObject             *object);
-static void terminal_window_destroy     (GtkObject           *object);
 
 static gboolean terminal_window_delete_event (GtkWidget *widget,
                                               GdkEvent *event,
@@ -115,27 +114,21 @@ static void       screen_set_menuitem  (TerminalScreen *screen,
                                         GtkWidget      *menuitem);
 static GtkWidget* screen_get_menuitem  (TerminalScreen *screen);
 
-static TerminalScreen* find_screen (TerminalWindow *window,
-                                    TerminalScreen *screen);
-
-static void notebook_page_selected_callback (GtkWidget *notebook,
-                                             GtkNotebookPage *useless_crap,
-                                             int              page_num,
-                                             TerminalWindow  *window);
-
-static void notebook_tab_added_callback     (GtkWidget       *notebook,
-                                             TerminalScreen  *screen,
-                                             TerminalWindow  *window);
-
-static void notebook_tab_detached_callback  (GtkWidget       *notebook,
-                                             TerminalScreen  *screen,
-                                             TerminalWindow  *window);
-
-static void notebook_tab_removed_callback   (GtkWidget       *notebook,
-                                             TerminalScreen  *screen,
-                                             TerminalWindow  *window);
-
-static void notebook_tabs_reordered_callback (GtkWidget      *notebook,
+static void notebook_page_selected_callback  (GtkWidget       *notebook,
+                                              GtkNotebookPage *useless_crap,
+                                              guint            page_num,
+                                              TerminalWindow  *window);
+static void notebook_page_added_callback     (GtkWidget       *notebook,
+                                              TerminalScreen  *screen,
+                                              guint            page_num,
+                                              TerminalWindow  *window);
+static void notebook_page_removed_callback   (GtkWidget       *notebook,
+                                              TerminalScreen  *screen,
+                                              guint            page_num,
+                                              TerminalWindow  *window);
+static void notebook_page_reordered_callback (GtkWidget      *notebook,
+                                              GtkWidget      *child,
+                                              guint           page_num,
                                               TerminalWindow *window);
 
 static void config_change_notify            (GConfClient *client,
@@ -143,14 +136,6 @@ static void config_change_notify            (GConfClient *client,
                                              GConfEntry  *entry,
                                              gpointer     user_data);
 
-static void menuitem_icon_visibility_notify (GConfClient *client,
-                                             guint        cnxn_id,
-                                             GConfEntry  *entry,
-                                             gpointer     user_data);
-
-static void menuitem_icon_visibility        (GtkWidget *item,
-                                             gboolean   use_image);
-
 static void reset_menubar_labels          (TerminalWindow *window);
 static void reset_tab_menuitems           (TerminalWindow *window);
 
@@ -243,36 +228,29 @@ static void terminal_window_show (GtkWidget *widget);
 
 static gboolean confirm_close_window (TerminalWindow *window);
 
-static gpointer parent_class;
+G_DEFINE_TYPE (TerminalWindow, terminal_window, GTK_TYPE_WINDOW)
 
-GType
-terminal_window_get_type (void)
+static GtkWidget*
+append_menuitem_common (GtkWidget  *menu,
+                        GtkWidget  *menu_item,
+                        const char *accel_path,
+                        GCallback   callback,
+                        gpointer    data)
 {
-  static GType object_type = 0;
+  if (accel_path)
+    gtk_menu_item_set_accel_path (GTK_MENU_ITEM (menu_item),
+                                  accel_path);
+  
+  gtk_widget_show (menu_item);
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu),
+                         menu_item);
 
-  g_type_init ();
-  
-  if (!object_type)
-    {
-      static const GTypeInfo object_info =
-      {
-        sizeof (TerminalWindowClass),
-        (GBaseInitFunc) NULL,
-        (GBaseFinalizeFunc) NULL,
-        (GClassInitFunc) terminal_window_class_init,
-        NULL,           /* class_finalize */
-        NULL,           /* class_data */
-        sizeof (TerminalWindow),
-        0,              /* n_preallocs */
-        (GInstanceInitFunc) terminal_window_init,
-      };
-      
-      object_type = g_type_register_static (GTK_TYPE_WINDOW,
-                                            "TerminalWindow",
-                                            &object_info, 0);      
-    }
-  
-  return object_type;
+  if (callback)
+    g_signal_connect (G_OBJECT (menu_item),
+                      "activate",
+                      callback, data);
+
+  return menu_item;
 }
 
 static GtkWidget*
@@ -282,108 +260,25 @@ append_menuitem (GtkWidget  *menu,
                  GCallback   callback,
                  gpointer    data)
 {
-  GtkWidget *menu_item;
-  
-  menu_item = gtk_menu_item_new_with_mnemonic (text);
-
-  if (accel_path)
-    gtk_menu_item_set_accel_path (GTK_MENU_ITEM (menu_item),
-                                  accel_path);
-  
-  gtk_widget_show (menu_item);
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu),
-                         menu_item);
-
-  if (callback)
-    g_signal_connect (G_OBJECT (menu_item),
-                      "activate",
-                      callback, data);
-
-  return menu_item;
-}
-
-static void
-menuitem_icon_visibility (GtkWidget *item, gboolean use_image)
-{
-  GtkWidget *image;
-
-  image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (item));
-  if (image && !g_object_get_data (G_OBJECT (item), "saved-image"))
-    g_object_set_data_full (G_OBJECT (item), "saved-image", g_object_ref (image), g_object_unref);
-
-  if (!image)
-    image = g_object_get_data (G_OBJECT (item), "saved-image");
-
-  if (!image)
-    return;
-
-  if (use_image)
-    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-  else
-    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), NULL);
-}
-
-static void
-menuitem_icon_visibility_notify (GConfClient *client,
-                                 guint        cnxn_id,
-                                 GConfEntry  *entry,
-                                 gpointer     user_data)
-{
-  GConfValue *value = gconf_entry_get_value (entry);
-  
-  if (value && value->type == GCONF_VALUE_BOOL)
-    menuitem_icon_visibility (user_data, gconf_value_get_bool (value));
-}
-
-void
-remove_notify (gpointer data)
-{
-  gconf_client_notify_remove (client, GPOINTER_TO_INT (data));
+  return append_menuitem_common (menu,
+                                 gtk_menu_item_new_with_mnemonic (text),
+                                 accel_path,
+                                 callback,
+                                 data);
 }
 
 static GtkWidget*
 append_stock_menuitem (GtkWidget  *menu,
-                       const char *text,
+                       const char *stock_id,
                        const char *accel_path,
                        GCallback   callback,
                        gpointer    data)
 {
-  GtkWidget *menu_item;
-  GtkWidget *image;
-  GError *error;
-
-  menu_item = gtk_image_menu_item_new_from_stock (text, NULL);
-  image = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (menu_item));
-
-  error = NULL;
-  notify = 0;
-
-  notify = gconf_client_notify_add (client, "/desktop/gnome/interface/menus_have_icons",
-                           menuitem_icon_visibility_notify,
-                           menu_item,
-                           NULL, &error);
-  if (error)
-    {
-      g_printerr (_("There was an error subscribing to notification of menu icon visibility changes. (%s)\n"), error->message);
-      g_error_free (error);
-    }
-
-  g_object_set_data_full (G_OBJECT (menu_item), "notify-id", GINT_TO_POINTER (notify), remove_notify);
-
-  if (accel_path)
-    gtk_menu_item_set_accel_path (GTK_MENU_ITEM (menu_item),
-                                  accel_path);
-
-  gtk_widget_show (menu_item);
-  gtk_menu_shell_append (GTK_MENU_SHELL (menu),
-                         menu_item);
-
-  if (callback)
-    g_signal_connect (G_OBJECT (menu_item),
-                      "activate",
-                      callback, data);
-
-  return menu_item;
+  return append_menuitem_common (menu,
+                                 gtk_image_menu_item_new_from_stock (stock_id, NULL),
+                                 accel_path,
+                                 callback,
+                                 data);
 }
 
 static GtkWidget *
@@ -784,6 +679,55 @@ terminal_window_uses_argb_visual (TerminalWindow *window)
   return window->priv->have_argb_visual;
 }
 
+static void
+update_tab_visibility (TerminalWindow *window,
+                        int             change)
+{
+  gboolean show_tabs;
+  guint num;
+
+  num = gtk_notebook_get_n_pages (GTK_NOTEBOOK (window->priv->notebook));
+
+  show_tabs = (num + change) > 1;
+  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (window->priv->notebook), show_tabs);
+}
+
+static GtkNotebook *
+handle_tab_droped_on_desktop (GtkNotebook *source_notebook,
+                              GtkWidget   *child,
+                              gint         x,
+                              gint         y,
+                              gpointer     data)
+{
+  TerminalScreen *screen;
+  TerminalWindow *source_window;
+  TerminalWindow *dest_window;
+  double zoom;
+
+  screen = TERMINAL_SCREEN (child);
+  source_window = TERMINAL_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (source_notebook)));
+
+  g_return_val_if_fail (TERMINAL_IS_WINDOW (source_window), NULL);
+
+  zoom = terminal_screen_get_font_scale (screen);
+
+  dest_window = terminal_app_new_window (terminal_app_get (), NULL, NULL, NULL, -1);
+  dest_window->priv->present_on_insert = TRUE;
+
+  update_tab_visibility (source_window, -1);
+  update_tab_visibility (dest_window, +1);
+
+  return GTK_NOTEBOOK (dest_window->priv->notebook);
+}
+
+static void
+terminal_window_realized_callback (GtkWidget *window,
+				   gpointer   user_data)
+{
+  gdk_window_set_group (window->window, window->window);
+  g_signal_handlers_disconnect_by_func (window, terminal_window_realized_callback, NULL);
+}
+
 static void
 terminal_window_init (TerminalWindow *window)
 {
@@ -791,29 +735,67 @@ terminal_window_init (TerminalWindow *window)
   GtkWidget *menu;
   GtkAccelGroup *accel_group;
   GError *error;
-  gboolean menus_have_icons;
+  gboolean use_mnemonics;
+
+  window->priv = G_TYPE_INSTANCE_GET_PRIVATE (window, TERMINAL_TYPE_WINDOW, TerminalWindowPrivate);
 
   g_signal_connect (G_OBJECT (window), "delete_event", 
                     G_CALLBACK(terminal_window_delete_event),
                     NULL);
+  g_signal_connect (G_OBJECT (window), "realize",
+                    G_CALLBACK (terminal_window_realized_callback),
+                    NULL);
 
   gtk_window_set_title (GTK_WINDOW (window), _("Terminal"));
-  client = gconf_client_get_default();
+
+  gtk_notebook_set_window_creation_hook (handle_tab_droped_on_desktop, NULL, NULL);
   
-  window->priv = g_new0 (TerminalWindowPrivate, 1);
-  window->priv->terms = NULL;
+  window->priv->terms = 0;
   window->priv->active_term = NULL;
   window->priv->menubar = gtk_menu_bar_new ();
   window->priv->menubar_visible = FALSE;
   
   window->priv->main_vbox = gtk_vbox_new (FALSE, 0);
-  window->priv->notebook = terminal_notebook_new ();
+  window->priv->notebook = gtk_notebook_new ();
+  gtk_notebook_set_scrollable (GTK_NOTEBOOK (window->priv->notebook), TRUE);
+  gtk_notebook_set_show_border (GTK_NOTEBOOK (window->priv->notebook), FALSE);
+  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (window->priv->notebook), FALSE);
+  gtk_notebook_set_group_id (GTK_NOTEBOOK (window->priv->notebook), 1);
   
   window->priv->old_char_width = -1;
   window->priv->old_char_height = -1;
   window->priv->old_geometry_widget = NULL;
   
-  window->priv->use_mnemonics = TRUE;
+  window->priv->conf = gconf_client_get_default ();
+
+  error = NULL;
+  window->priv->notify_id =
+    gconf_client_notify_add (window->priv->conf,
+                             CONF_GLOBAL_PREFIX,
+                             config_change_notify,
+                             window,
+                             NULL, &error);
+  
+  if (error)
+    {
+      g_printerr (_("There was an error subscribing to notification of terminal window configuration changes. (%s)\n"),
+                  error->message);
+      g_error_free (error);
+    }
+
+  error = NULL;
+  use_mnemonics = gconf_client_get_bool (window->priv->conf,
+                                         CONF_GLOBAL_PREFIX"/use_mnemonics",
+                                         &error);
+  if (error)
+    {
+      g_printerr (_("There was an error loading config value for whether to use mnemonics. (%s)\n"),
+                  error->message);
+      g_error_free (error);
+      use_mnemonics = TRUE;
+    }
+  window->priv->use_mnemonics = use_mnemonics;
+
   window->priv->using_mnemonics = FALSE;
 
   initialize_alpha_mode (window);
@@ -821,15 +803,6 @@ terminal_window_init (TerminalWindow *window)
   /* force gtk to construct its GtkClipboard; otherwise our UI is very slow the first time we need it */
   window->priv->clipboard = gtk_clipboard_get_for_display (gtk_widget_get_display (GTK_WIDGET (window)), GDK_NONE);
 
-  error = NULL;
-  menus_have_icons = gconf_client_get_bool (client, "/desktop/gnome/interface/menus_have_icons", &error);
-  if (error)
-    {
-      g_printerr (_("There was an error loading config value for whether to use image in menus. (%s)\n"),error->message);
-      g_error_free (error);
-      menus_have_icons = TRUE;
-    }
- 
   g_signal_connect (window->priv->menubar,
 		    "can_activate_accel",
 		    G_CALLBACK (gtk_true),
@@ -842,28 +815,23 @@ terminal_window_init (TerminalWindow *window)
                                TRUE);                                      
   
   g_signal_connect_after (G_OBJECT (window->priv->notebook),
-                          "switch_page",
+                          "switch-page",
                           G_CALLBACK (notebook_page_selected_callback),
                           window);
 
   g_signal_connect_after (G_OBJECT (window->priv->notebook),
-                          "tab_added",
-                          G_CALLBACK (notebook_tab_added_callback),
+                          "page-added",
+                          G_CALLBACK (notebook_page_added_callback),
                           window);
 
   g_signal_connect_after (G_OBJECT (window->priv->notebook),
-                          "tab_detached",
-                          G_CALLBACK (notebook_tab_detached_callback),
-                          window);
-
-  g_signal_connect_after (G_OBJECT (window->priv->notebook),
-                          "tab_removed",
-                          G_CALLBACK (notebook_tab_removed_callback),
+                          "page-removed",
+                          G_CALLBACK (notebook_page_removed_callback),
                           window);
   
   g_signal_connect_after (G_OBJECT (window->priv->notebook),
-                          "tabs_reordered",
-                          G_CALLBACK (notebook_tabs_reordered_callback),
+                          "page-reordered",
+                          G_CALLBACK (notebook_page_reordered_callback),
                           window);
 
   gtk_container_add (GTK_CONTAINER (window),
@@ -929,15 +897,13 @@ terminal_window_init (TerminalWindow *window)
                            GTK_STOCK_COPY, ACCEL_PATH_COPY,
                            G_CALLBACK (copy_callback),
                            window);
-  menuitem_icon_visibility (window->priv->copy_menuitem, menus_have_icons);
 
   window->priv->paste_menuitem =
     append_stock_menuitem (menu,
                            GTK_STOCK_PASTE, ACCEL_PATH_PASTE,
                            G_CALLBACK (paste_callback),
                            window);
-  menuitem_icon_visibility (window->priv->paste_menuitem, menus_have_icons);
-  g_signal_connect (G_OBJECT (mi), "activate", 
+  g_signal_connect (G_OBJECT (mi), "activate",
                     G_CALLBACK (edit_menu_activate_callback), window);
 
   mi = gtk_separator_menu_item_new ();
@@ -982,17 +948,14 @@ terminal_window_init (TerminalWindow *window)
   mi = append_stock_menuitem (menu, GTK_STOCK_ZOOM_IN, ACCEL_PATH_ZOOM_IN,
                               G_CALLBACK (zoom_in_callback), window);
   window->priv->zoom_in_menuitem = mi;
-  menuitem_icon_visibility (window->priv->zoom_in_menuitem, menus_have_icons);
 
   mi = append_stock_menuitem (menu, GTK_STOCK_ZOOM_OUT, ACCEL_PATH_ZOOM_OUT,
                               G_CALLBACK (zoom_out_callback), window);
   window->priv->zoom_out_menuitem = mi;
-  menuitem_icon_visibility (window->priv->zoom_out_menuitem, menus_have_icons);
 
   mi = append_stock_menuitem (menu, GTK_STOCK_ZOOM_100, ACCEL_PATH_ZOOM_NORMAL,
                               G_CALLBACK (zoom_normal_callback), window);
   window->priv->zoom_normal_menuitem = mi;
-  menuitem_icon_visibility (window->priv->zoom_normal_menuitem, menus_have_icons);
 
   update_zoom_items (window);
   
@@ -1095,7 +1058,6 @@ terminal_window_init (TerminalWindow *window)
   mi = append_stock_menuitem (menu, GTK_STOCK_HELP, NULL,
 			      G_CALLBACK (help_callback), window);
   set_menuitem_text (mi, _("_Contents"), FALSE);
-  menuitem_icon_visibility (mi, menus_have_icons);
 
   gtk_accel_map_add_entry (ACCEL_PATH_HELP, GDK_F1, 0);
   gtk_menu_item_set_accel_path (GTK_MENU_ITEM (mi),
@@ -1105,7 +1067,6 @@ terminal_window_init (TerminalWindow *window)
   mi = append_stock_menuitem (menu, GTK_STOCK_ABOUT, NULL,
                               G_CALLBACK (about_callback), window);
   set_menuitem_text (mi, _("_About"), FALSE);
-  menuitem_icon_visibility (mi, menus_have_icons);
   
   terminal_window_reread_profile_list (window);
   
@@ -1113,7 +1074,8 @@ terminal_window_init (TerminalWindow *window)
   window->priv->use_default_menubar_visibility = TRUE;  
 
   reset_menubar_labels (window);
-  
+  reset_tab_menuitems (window);
+
   gtk_widget_show_all (window->priv->main_vbox);
 }
 
@@ -1121,16 +1083,44 @@ static void
 terminal_window_class_init (TerminalWindowClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  GtkObjectClass *gtk_object_class = GTK_OBJECT_CLASS (klass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
   
-  parent_class = g_type_class_peek_parent (klass);
-  
   object_class->finalize = terminal_window_finalize;
-  gtk_object_class->destroy = terminal_window_destroy;
+  object_class->dispose = terminal_window_dispose;
   widget_class->show = terminal_window_show;
+
+  g_type_class_add_private (object_class, sizeof (TerminalWindowPrivate));
+
+        
+  gtk_rc_parse_string ("style \"gnome-terminal-tab-close-button-style\"\n"
+                       "{\n"
+                          "GtkWidget::focus-padding = 0\n"
+                          "GtkWidget::focus-line-width = 0\n"
+                          "xthickness = 0\n"
+                          "ythickness = 0\n"
+                       "}\n"
+                       "widget \"*.gnome-terminal-tab-close-button\" style \"gnome-terminal-tab-close-button-style\"");
+
+
 }
 
+static void
+terminal_window_dispose (GObject *object)
+{
+  TerminalWindow *window = TERMINAL_WINDOW (object);
+  TerminalWindowPrivate *priv = window->priv;
+
+  if (priv->notify_id != 0)
+    {
+      gconf_client_notify_remove (priv->conf, priv->notify_id);
+      priv->notify_id = 0;
+    }
+
+  priv->disposed = TRUE;
+
+  G_OBJECT_CLASS (terminal_window_parent_class)->dispose (object);
+}
+   
 static void
 terminal_window_finalize (GObject *object)
 {
@@ -1138,44 +1128,24 @@ terminal_window_finalize (GObject *object)
 
   window = TERMINAL_WINDOW (object);
 
-  gconf_client_notify_remove (window->priv->conf,
-                              window->priv->notify_id);
-  window->priv->notify_id = 0;
-
   if (window->priv->conf)
-    g_object_unref (G_OBJECT (window->priv->conf));
+    {
+      g_object_unref (G_OBJECT (window->priv->conf));
+    }
   
   if (window->priv->icon)
-    g_object_unref (G_OBJECT (window->priv->icon));
+    {
+      g_object_unref (G_OBJECT (window->priv->icon));
+    }
+
+  if (window->priv->tab_menuitems)
+    {
+      g_list_free (window->priv->tab_menuitems);
+    }
 
   g_free (window->priv->startup_id);
-  
-  g_free (window->priv);
-  
-  G_OBJECT_CLASS (parent_class)->finalize (object);
-}
 
-static void
-terminal_window_destroy (GtkObject *object)
-{
-  TerminalWindow *window;
-
-  window = TERMINAL_WINDOW (object);
-
-  g_list_free (window->priv->tab_menuitems);
-  window->priv->tab_menuitems = NULL;
-  
-  /* to handle multiple destroy, FIXME some fields are missing */
-  window->priv->menubar = NULL;
-  window->priv->notebook = NULL;
-  window->priv->main_vbox = NULL;
-  window->priv->copy_menuitem = NULL;
-  window->priv->paste_menuitem = NULL;
-  window->priv->edit_config_menuitem = NULL;
-  window->priv->choose_config_menuitem = NULL;
-  window->priv->encoding_menuitem = NULL;
-  
-  GTK_OBJECT_CLASS (parent_class)->destroy (object);  
+  G_OBJECT_CLASS (terminal_window_parent_class)->finalize (object);
 }
 
 static gboolean
@@ -1254,7 +1224,7 @@ terminal_window_show (GtkWidget *widget)
         }
     }
   
-  GTK_WIDGET_CLASS (parent_class)->show (widget);
+  GTK_WIDGET_CLASS (terminal_window_parent_class)->show (widget);
 
   if (context != NULL)
     {
@@ -1264,60 +1234,10 @@ terminal_window_show (GtkWidget *widget)
     }
 }
 
-void terminal_window_realized_callback (GtkWidget *window,
-                                        gpointer   user_data)
-{
-  gdk_window_set_group (window->window, window->window);
-  g_signal_handlers_disconnect_by_func (window, terminal_window_realized_callback, NULL);
-}
-
 TerminalWindow*
-terminal_window_new (GConfClient *conf)
+terminal_window_new (void)
 {
-  TerminalWindow *window;
-  GError *err;
-  gboolean use_mnemonics;
-  
-  window = g_object_new (TERMINAL_TYPE_WINDOW, NULL);
-  g_signal_connect (G_OBJECT (window), "realize", G_CALLBACK (terminal_window_realized_callback), NULL);
-
-  window->priv->conf = conf;
-  g_object_ref (G_OBJECT (conf));
-
-  err = NULL;
-  use_mnemonics = gconf_client_get_bool (window->priv->conf,
-                                         CONF_GLOBAL_PREFIX"/use_mnemonics",
-                                         &err);
-  if (err)
-    {
-      g_printerr (_("There was an error loading config value for whether to use mnemonics. (%s)\n"),
-                  err->message);
-      g_error_free (err);
-    }
-  else
-    {
-      window->priv->use_mnemonics = use_mnemonics;
-    }
-  
-  err = NULL;
-  window->priv->notify_id =
-    gconf_client_notify_add (window->priv->conf,
-                             CONF_GLOBAL_PREFIX,
-                             config_change_notify,
-                             window,
-                             NULL, &err);
-  
-  if (err)
-    {
-      g_printerr (_("There was an error subscribing to notification of terminal window configuration changes. (%s)\n"),
-                  err->message);
-      g_error_free (err);
-    }
-
-  reset_menubar_labels (window);
-  reset_tab_menuitems (window);
-  
-  return window;
+  return g_object_new (TERMINAL_TYPE_WINDOW, NULL);
 }
 
 static void
@@ -1325,7 +1245,7 @@ update_notebook (TerminalWindow *window)
 {
   gboolean single;
 
-  single = g_list_length (window->priv->terms) == 1;
+  single = window->priv->terms == 1;
     
   gtk_notebook_set_show_border (GTK_NOTEBOOK (window->priv->notebook), !single);
 }
@@ -1404,6 +1324,12 @@ update_tab_sensitivity (TerminalWindow *window)
   gboolean on_last_page;
 
   notebook = window->priv->notebook;
+
+  if (notebook == NULL)
+    {
+      return;
+    }
+
   num_pages = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook));
   page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
   
@@ -1441,11 +1367,92 @@ selection_changed_callback (TerminalScreen *screen,
   update_copy_sensitivity (window);
 }
 
+static void
+close_button_clicked_cb (GtkWidget *widget, GtkWidget *screen)
+{
+  GtkWidget *notebook;
+  guint page_num;
+
+  notebook = gtk_widget_get_parent (GTK_WIDGET (screen));
+  page_num = gtk_notebook_page_num (GTK_NOTEBOOK (notebook), screen);
+  gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page_num);
+}
+
+static void
+sync_tab_label (TerminalScreen *screen,
+                GtkWidget *label)
+{
+  GtkWidget *hbox;
+  const char *title;
+
+  title = terminal_screen_get_title (screen);
+
+  hbox = gtk_widget_get_parent (label);
+
+  gtk_label_set_text (GTK_LABEL (label), title);
+  gtk_widget_set_tooltip_text (hbox, title);
+}
+
+static void
+tab_label_style_set_cb (GtkWidget *hbox,
+                        GtkStyle *previous_style,
+                        GtkWidget *button)
+{
+        int h, w;
+
+        gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (button),
+                                           GTK_ICON_SIZE_MENU, &w, &h);
+        gtk_widget_set_size_request (button, w + 2, h + 2);
+}
+
+static GtkWidget *
+contruct_tab_label (TerminalWindow *window, TerminalScreen *screen)
+{
+  GtkWidget *hbox, *label, *close_button, *image;
+
+  hbox = gtk_hbox_new (FALSE, 4);
+
+  label = gtk_label_new (NULL);
+  gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+  gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
+
+  close_button = gtk_button_new ();
+  gtk_button_set_relief (GTK_BUTTON (close_button), GTK_RELIEF_NONE);
+  gtk_button_set_focus_on_click (GTK_BUTTON (close_button), FALSE);
+  gtk_button_set_relief (GTK_BUTTON (close_button), GTK_RELIEF_NONE);
+  gtk_widget_set_name (close_button, "gnome-terminal-tab-close-button");
+  gtk_widget_set_tooltip_text (close_button, _("Close tab"));
+
+  image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
+  gtk_container_add (GTK_CONTAINER (close_button), image);
+  gtk_box_pack_start (GTK_BOX (hbox), close_button, FALSE, FALSE, 0);
+
+  g_signal_connect (G_OBJECT (close_button), "clicked",
+		    G_CALLBACK (close_button_clicked_cb),
+		    screen);
+
+  sync_tab_label (screen, label);
+  g_signal_connect (G_OBJECT (screen),
+                    "title-changed",
+                    G_CALLBACK (sync_tab_label),
+                    label);
+
+  g_signal_connect (G_OBJECT (hbox), "style-set",
+                    G_CALLBACK (tab_label_style_set_cb), close_button);
+
+  gtk_widget_show_all (hbox);
+
+  return hbox;
+}
+
 void
 terminal_window_add_screen (TerminalWindow *window,
-                            TerminalScreen *screen)
+                            TerminalScreen *screen,
+                            gint            position)
 {
   TerminalWindow *old;
+  GtkWidget *tab_label;
  
   old = terminal_screen_get_window (screen);
 
@@ -1457,46 +1464,54 @@ terminal_window_add_screen (TerminalWindow *window,
   if (old)
     terminal_window_remove_screen (old, screen);
 
-  /* keep this list in same order as tabs in notebook,
-   * so that terminal_window_list_screens() returns screens
-   * in tab order
-   */
-  window->priv->terms = g_list_append (window->priv->terms, screen);
-
   terminal_screen_set_window (screen, window);
 
   gtk_widget_show_all (GTK_WIDGET (screen));
 
-  terminal_notebook_add_tab (TERMINAL_NOTEBOOK (window->priv->notebook),
-                             screen,
-                             -1,
-                             FALSE);
+  update_tab_visibility (window, +1);
+
+  tab_label = contruct_tab_label (window, screen);
+
+  gtk_notebook_insert_page (GTK_NOTEBOOK (window->priv->notebook),
+                            GTK_WIDGET (screen),
+                            tab_label,
+                            position);
+  gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (window->priv->notebook),
+                                      GTK_WIDGET (screen),
+                                      TRUE, TRUE, GTK_PACK_START);
+  gtk_notebook_set_tab_reorderable (GTK_NOTEBOOK (window->priv->notebook),
+                                    GTK_WIDGET (screen),
+                                    TRUE);
+  gtk_notebook_set_tab_detachable (GTK_NOTEBOOK (window->priv->notebook),
+                                   GTK_WIDGET (screen),
+                                   TRUE);
 }
 
 void
 terminal_window_remove_screen (TerminalWindow *window,
                                TerminalScreen *screen)
 {
+  guint num_page;
+
   g_return_if_fail (terminal_screen_get_window (screen) == window);
 
-  window->priv->terms = g_list_remove (window->priv->terms, screen);
-  
-  terminal_screen_set_window (screen, NULL);
-  
-  terminal_notebook_remove_tab (TERMINAL_NOTEBOOK (window->priv->notebook),
-                                screen);
+  update_tab_visibility (window, -1);
+
+  num_page = gtk_notebook_page_num (GTK_NOTEBOOK (window->priv->notebook), GTK_WIDGET (screen));
+  gtk_notebook_remove_page (GTK_NOTEBOOK (window->priv->notebook), num_page);
 }
 
 GList*
 terminal_window_list_screens (TerminalWindow *window)
 {
-  return g_list_copy (window->priv->terms);
+  /* We are trusting that GtkNotebook will return pages in order */
+  return gtk_container_get_children (GTK_CONTAINER (window->priv->notebook));
 }
 
 int    
 terminal_window_get_screen_count (TerminalWindow *window)
 {
-  return g_list_length (window->priv->terms);
+  return window->priv->terms;
 }
 
 void
@@ -1626,6 +1641,7 @@ terminal_window_set_active (TerminalWindow *window,
 {
   GtkWidget *widget;
   TerminalProfile *profile;
+  guint page_num;
   
   if (window->priv->active_term == screen)
     return;
@@ -1668,9 +1684,8 @@ terminal_window_set_active (TerminalWindow *window,
   gdk_window_set_icon_name (GTK_WIDGET (window)->window, terminal_screen_get_icon_title (screen));
   gtk_window_set_title (GTK_WINDOW (window), terminal_screen_get_title (screen));
 
-  gtk_notebook_set_current_page (GTK_NOTEBOOK (window->priv->notebook),
-                                 gtk_notebook_page_num (GTK_NOTEBOOK (window->priv->notebook),
-                                                        GTK_WIDGET (screen)));
+  page_num = gtk_notebook_page_num (GTK_NOTEBOOK (window->priv->notebook), GTK_WIDGET (screen));
+  gtk_notebook_set_current_page (GTK_NOTEBOOK (window->priv->notebook), page_num);
 
   /* set size of window to current grid size */
 #ifdef DEBUG_GEOMETRY
@@ -1708,28 +1723,10 @@ screen_get_menuitem (TerminalScreen *screen)
   return g_object_get_data (G_OBJECT (screen), "menuitem");
 }
 
-static TerminalScreen*
-find_screen (TerminalWindow *window,
-             TerminalScreen *screen)
-{
-  GList *tmp;
-
-  tmp = window->priv->terms;
-  while (tmp != NULL)
-    {
-      if (tmp->data == screen)
-        return tmp->data;
-
-      tmp = tmp->next;
-    }
-
-  return NULL;
-}
-
 static void
 notebook_page_selected_callback (GtkWidget       *notebook,
                                  GtkNotebookPage *useless_crap,
-                                 int              page_num,
+                                 guint            page_num,
                                  TerminalWindow  *window)
 {
   GtkWidget* page_widget;
@@ -1738,7 +1735,7 @@ notebook_page_selected_callback (GtkWidget       *notebook,
   int old_grid_width, old_grid_height;
   GtkWidget *old_widget, *new_widget;
 
-  if (window->priv->active_term == NULL)
+  if (window->priv->active_term == NULL || window->priv->disposed)
     return;
 
   old_widget = terminal_screen_get_widget (window->priv->active_term);
@@ -1769,27 +1766,18 @@ notebook_page_selected_callback (GtkWidget       *notebook,
 }
 
 static void
-notebook_tab_added_callback (GtkWidget       *notebook,
-                               TerminalScreen  *screen,
-                               TerminalWindow  *window)
+notebook_page_added_callback (GtkWidget       *notebook,
+                              TerminalScreen  *screen,
+                              guint            page_num,
+                              TerminalWindow  *window)
 {
   GtkWidget *term;
 
-  /* Moved from another window */
-  if (find_screen (window, screen) == NULL) {
-    g_object_ref (G_OBJECT (screen)); /* make our own new refcount */
-  
-    /* keep this list in same order as tabs in notebook,
-     * so that terminal_window_list_screens() returns screens
-     * in tab order
-     */
-    window->priv->terms = g_list_append (window->priv->terms, screen);
-  
-    terminal_screen_set_window (screen, window);
-  }
-  
+  terminal_screen_set_window (screen, window);
+  window->priv->terms++;
+
   g_signal_connect (G_OBJECT (screen),
-                    "profile_set",
+                    "profile-set",
                     G_CALLBACK (profile_set_callback),
                     window);
 
@@ -1814,7 +1802,8 @@ notebook_tab_added_callback (GtkWidget       *notebook,
 
   reset_tab_menuitems (window);
   update_tab_sensitivity (window);
-  
+  update_tab_visibility (window, 0);
+
   term = terminal_screen_get_widget (screen);
 
   /* ZvtTerm is a broken POS and requires this realize to get
@@ -1840,34 +1829,25 @@ notebook_tab_added_callback (GtkWidget       *notebook,
   /* Make the first-added screen the active one */
   if (window->priv->active_term == NULL)
     terminal_window_set_active (window, screen);
+
+  if (window->priv->present_on_insert)
+    {
+      gtk_widget_show_all (GTK_WIDGET (window));
+      window->priv->present_on_insert = FALSE;
+    }
 }
 
 static void
-notebook_tab_detached_callback (GtkWidget       *notebook,
+notebook_page_removed_callback (GtkWidget       *notebook,
                                 TerminalScreen  *screen,
+                                guint            page_num,
                                 TerminalWindow  *window)
 {
-  /* Reset the tab menu before detaching the tab, in case the tab changed
-   * position before it was detached (bug #330246). */
-  reset_tab_menuitems (window);
+  int pages;
 
-  detach_tab (screen, window);
-}
+  if (window->priv->disposed)
+    return;
 
-static void
-notebook_tab_removed_callback (GtkWidget       *notebook,
-                               TerminalScreen  *screen,
-                               TerminalWindow  *window)
-{  
-  gboolean single;
-  /* Called from terminal_notebook_move_tab() */
-  if (find_screen (window, screen) != NULL) {
-    g_assert (terminal_screen_get_window (screen) == window);
-  
-    window->priv->terms = g_list_remove (window->priv->terms, screen);
-    terminal_screen_set_window (screen, NULL);
-  }
-  
   g_signal_handlers_disconnect_by_func (G_OBJECT (screen),
                                         G_CALLBACK (profile_set_callback),
                                         window);
@@ -1884,29 +1864,38 @@ notebook_tab_removed_callback (GtkWidget       *notebook,
                                         G_CALLBACK (selection_changed_callback),
                                         window);
 
+  terminal_screen_set_window (screen, NULL);
+  window->priv->terms--;
+
   screen_set_menuitem (screen, NULL);
   
   update_notebook (window);
 
   reset_tab_menuitems (window);
   update_tab_sensitivity (window);
+  update_tab_visibility (window, 0);
 
-  /* The tab bar may have disappeared */
-  single = g_list_length (window->priv->terms) == 1;
-  if (single)
-    terminal_window_set_size (window, window->priv->active_term, TRUE);
-
-  /* Close window if no more terminals */
-  if (window->priv->terms == NULL)
-    gtk_widget_destroy (GTK_WIDGET (window));
+  pages = terminal_window_get_screen_count (window);
+  if (pages == 1)
+    {
+      terminal_window_set_size (window, window->priv->active_term, TRUE);
+    }
+  else if (pages == 0)
+    {
+      gtk_widget_destroy (GTK_WIDGET (window));
+    }
 }
 
 static void
-notebook_tabs_reordered_callback (GtkWidget       *notebook,
+notebook_page_reordered_callback (GtkWidget       *notebook,
+                                  GtkWidget       *child,
+                                  guint            page_num,
                                   TerminalWindow  *window)
 {
-	reset_tab_menuitems (window);
-        update_tab_sensitivity (window);
+  if (window->priv->disposed)
+    return;
+
+  reset_tab_menuitems(window);
 }
 
 void
@@ -2776,12 +2765,8 @@ move_left_tab_callback(GtkWidget      *menuitem,
   last_page = gtk_notebook_get_n_pages (notebook) - 1;
   page = gtk_notebook_get_nth_page (notebook, page_num);
   
-  if (page_num == 0)
-    terminal_notebook_move_tab (TERMINAL_NOTEBOOK (notebook), NULL,
-                                TERMINAL_SCREEN (page), last_page);
-  else
-    terminal_notebook_move_tab (TERMINAL_NOTEBOOK (notebook), NULL,
-                                TERMINAL_SCREEN (page), page_num-1);
+
+  gtk_notebook_reorder_child (notebook, page, page_num == 0 ? last_page : page_num - 1);
 
   update_tab_sensitivity (window);
   reset_tab_menuitems (window);
@@ -2800,12 +2785,7 @@ move_right_tab_callback(GtkWidget      *menuitem,
   last_page = gtk_notebook_get_n_pages (notebook) - 1;
   page = gtk_notebook_get_nth_page (notebook, page_num);
   
-  if (page_num == last_page)
-    terminal_notebook_move_tab (TERMINAL_NOTEBOOK (notebook), NULL,
-                                TERMINAL_SCREEN (page), 0);
-  else
-    terminal_notebook_move_tab (TERMINAL_NOTEBOOK (notebook), NULL,
-                                TERMINAL_SCREEN (page), page_num+1);
+  gtk_notebook_reorder_child (notebook, page, page_num == last_page ? 0 : page_num + 1);
 
   update_tab_sensitivity (window);
   reset_tab_menuitems (window);
